// This file is part of VKF, a CPython library
// for Machine Learning based on Lattice Theory.
//
// Copyright (c) 2020 Dmitry V. Vinogradov
//
// This Source Code Form is subject to the terms of the X11 (MIT)
// License. If a copy of the X11 License was not distributed
// with this file, You can obtain one at https://opensource.org/licenses/MIT/.

#ifndef VKF_DATA_HPP
#define VKF_DATA_HPP

#define DEFAULT_HOST "localhost"
#define DEFAULT_PASS "toor"
#define DEFAULT_USER "root"

#include <mariadb++/account.hpp>
#include <mariadb++/connection.hpp>

#include <string>

#include "Encoder.hpp"

namespace vkf {

class CData {
    public:
        /** default constructor */
        CData() = delete;
        /** constructor for continuous attributes only */
        CData(  const std::string &data_table_name,
                const std::string &verges_table_name,
                const std::string &database,
                const std::string &url = DEFAULT_HOST,
                const std::string &user = DEFAULT_USER,
                const std::string &password = DEFAULT_PASS
                ) {
            std::list<std::string> temp;
            // set up the account
            mariadb::account_ref acc = mariadb::account::create(url, user, password);
            // create connection
            mariadb::connection_ref con = mariadb::connection::create(acc);
            // create table sample
            con->execute("USE " + database);
            mariadb::result_set_ref result = con->query("SELECT * FROM " + verges_table_name);
            // reading from a result set
            while (result->next()) {
                std::string attr_name = result->get_string("attribute_name"); /**< This variable contains a name of the attribute */
                if (attr_name.find("__") == std::string::npos) /** test originality of attribute */ {
                    auto it = std::find_if(temp.begin(), temp.end(), [&](const std::string &arg){ return arg == attr_name; });
                    if (it == temp.end()) {
                        temp.push_back(attr_name);
                    }
                }
            }
            con->execute("DROP TABLE IF EXISTS " + data_table_name);
            std::string sql_query("CREATE TABLE IF NOT EXISTS " + data_table_name);
            sql_query.append(" (row_id int(8) NOT NULL AUTO_INCREMENT, ");
            sql_query.append("real_sign tinyint(1) NOT NULL DEFAULT 1, ");
            for (auto name : temp) {
                sql_query.append(name);
                sql_query.append(" float(53) NOT NULL DEFAULT 0E0, ");
            }
            sql_query.append("PRIMARY KEY(row_id)) ");
            sql_query.append("ENGINE = InnoDB");
            con->execute(sql_query);
            con->~connection();
            acc->~account();
        }
        /** default destructor */
        ~CData() = default;
};

class DData {
    public:
        /** default constructor */
        DData() = delete;
        /** constructor for discrete attributes only */
        DData(  const std::string &data_table_name,
                const std::string &encoder_table_name,
                const std::string &database,
                const std::string &url = DEFAULT_HOST,
                const std::string &user = DEFAULT_USER,
                const std::string &password = DEFAULT_PASS
                ) {
            Encoder encoder(encoder_table_name, database, url, user, password);
            // set up the account
            mariadb::account_ref acc = mariadb::account::create(url, user, password);
            // create connection
            mariadb::connection_ref con = mariadb::connection::create(acc);
            // create table sample
            con->execute("USE " + database);
            con->execute("DROP TABLE IF EXISTS " + data_table_name);
            std::string sql_query("CREATE TABLE IF NOT EXISTS " + data_table_name);
            sql_query.append(" (row_id int(8) NOT NULL AUTO_INCREMENT, ");
            sql_query.append("real_sign tinyint(1) NOT NULL DEFAULT 1, ");
            for (auto maps : encoder) {
                sql_query.append(maps.getAttributeName());
                sql_query.append(" varchar(255) NOT NULL DEFAULT 'null', ");
            }
            sql_query.append("PRIMARY KEY(row_id)) ");
            sql_query.append("ENGINE = InnoDB");
//            std::cout << sql_query << std::endl;
            con->execute(sql_query);
            con->~connection();
            acc->~account();
        }
        /** default destructor */
        ~DData() = default;
};

class FData {
    public:
        /** default constructor */
        FData() = delete;
        /** constructor for discrete attributes only */
        FData(  const std::string &data_table_name,
                const std::string &encoder_table_name,
                const std::string &verges_table_name,
                const std::string &database,
                const std::string &url = DEFAULT_HOST,
                const std::string &user = DEFAULT_USER,
                const std::string &password = DEFAULT_PASS
                ) {
            Encoder encoder(encoder_table_name, database, url, user, password);
                        std::list<std::string> temp;
            // set up the account
            mariadb::account_ref acc = mariadb::account::create(url, user, password);
            // create connection
            mariadb::connection_ref con = mariadb::connection::create(acc);
            // create table sample
            con->execute("USE " + database);
            mariadb::result_set_ref result = con->query("SELECT * FROM " + verges_table_name);
            // reading from a result set
            while (result->next()) {
                std::string attr_name = result->get_string("attribute_name"); /**< This variable contains a name of the attribute */
                if (attr_name.find("__") == std::string::npos) /** test originality of attribute */ {
                    auto it = std::find_if(temp.begin(), temp.end(), [&](const std::string &arg){ return arg == attr_name; });
                    if (it == temp.end()) {
                        temp.push_back(attr_name);
                    }
                }
            }
            con->execute("USE " + database);
            con->execute("DROP TABLE IF EXISTS " + data_table_name);
            std::string sql_query("CREATE TABLE IF NOT EXISTS " + data_table_name);
            sql_query.append(" (row_id int(8) NOT NULL AUTO_INCREMENT, ");
            sql_query.append("real_sign tinyint(1) NOT NULL DEFAULT 1, ");
            for (auto maps : encoder) {
                sql_query.append(maps.getAttributeName());
                sql_query.append(" varchar(255) NOT NULL DEFAULT 'null', ");
            }
            for (auto name : temp) {
                sql_query.append(name);
                sql_query.append(" float(53) NOT NULL DEFAULT 0E0, ");
            }
            sql_query.append("PRIMARY KEY(row_id)) ");
            sql_query.append("ENGINE = InnoDB");
//            std::cout << sql_query << std::endl;
            con->execute(sql_query);
            con->~connection();
            acc->~account();
        }
        /** default destructor */
        ~FData() = default;
};

} // namespace vkf
#endif // VKF_DATA_HPP
